//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef PROCESSOR_POINT_FEET_NOMOVE_H
#define PROCESSOR_POINT_FEET_NOMOVE_H

// Wolf
#include "core/sensor/sensor_base.h"
#include "core/processor/processor_base.h"

#include "bodydynamics/capture/capture_point_feet_nomove.h"
#include "bodydynamics/sensor/sensor_point_feet_nomove.h"
#include "bodydynamics/factor/factor_point_feet_nomove.h"
#include "bodydynamics/factor/factor_point_feet_altitude.h"

namespace wolf {
WOLF_STRUCT_PTR_TYPEDEFS(ParamsProcessorPointFeetNomove);

struct ParamsProcessorPointFeetNomove : public ParamsProcessorBase
{
    double max_time_vote = -1;

    ParamsProcessorPointFeetNomove() = default;
    ParamsProcessorPointFeetNomove(std::string _unique_name, const ParamsServer& _server):
        ParamsProcessorBase(_unique_name, _server)
    {
        max_time_vote = _server.getParam<double>(_unique_name + "/max_time_vote");
    }
    ~ParamsProcessorPointFeetNomove() override = default;
    std::string print() const override
    {
        return "\n" + ParamsProcessorBase::print() + "\n"
                    + "max_time_vote: " + std::to_string(max_time_vote) + "\n";    
    }
};

WOLF_PTR_TYPEDEFS(ProcessorPointFeetNomove);
    
//class
class ProcessorPointFeetNomove : public ProcessorBase{
    public:
        ProcessorPointFeetNomove(ParamsProcessorPointFeetNomovePtr _params_pfnomove);
        ~ProcessorPointFeetNomove() override = default;
        WOLF_PROCESSOR_CREATE(ProcessorPointFeetNomove, ParamsProcessorPointFeetNomove);

        void configure(SensorBasePtr _sensor) override;

        bool createPointFeetNomoveFactor(CapturePointFeetNomovePtr _cap_pfnomove, CaptureBasePtr _cap_angvel, CaptureBasePtr _cap_pfnomove_origin);

        void processCapture(CaptureBasePtr _incoming) override;
        void processKeyFrame(FrameBasePtr _keyframe_ptr) override;
        bool triggerInCapture(CaptureBasePtr) const override;
        bool triggerInKeyFrame(FrameBasePtr _keyframe_ptr) const override;
        bool storeKeyFrame(FrameBasePtr) override;
        bool storeCapture(CaptureBasePtr) override;
        bool voteForKeyFrame() const override;

        void createFactorConsecutive();

        void processCaptureCreateFactorFar(CaptureBasePtr _capture);
        void processKeyFrameCreateFactorFar();
        


    protected:
        ParamsProcessorPointFeetNomovePtr params_pfnomove_;
        CaptureBasePtr origin_;
        CaptureBasePtr incoming_;
        std::map<int, CaptureBasePtr> tracks_;
};



inline bool ProcessorPointFeetNomove::triggerInKeyFrame(FrameBasePtr _keyframe_ptr) const
{
    return true;
}

inline bool ProcessorPointFeetNomove::triggerInCapture(CaptureBasePtr _capture) const
{   
    return true;
}

inline bool ProcessorPointFeetNomove::storeKeyFrame(FrameBasePtr)
{
    return true;
}

inline bool ProcessorPointFeetNomove::storeCapture(CaptureBasePtr)
{
    return true;
}


} /* namespace wolf */

/////////////////////////////////////////////////////////
// IMPLEMENTATION. Put your implementation includes here
/////////////////////////////////////////////////////////


namespace wolf{

} // namespace wolf

#endif // PROCESSOR_POINT_FEET_NOMOVE_H
