//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef SENSOR_FORCE_TORQUE_H
#define SENSOR_FORCE_TORQUE_H

//wolf includes
#include "core/sensor/sensor_base.h"
#include "core/utils/params_server.h"
#include <iostream>

namespace wolf {

WOLF_STRUCT_PTR_TYPEDEFS(ParamsSensorForceTorque);


struct ParamsSensorForceTorque : public ParamsSensorBase
{
        //noise std dev
        double mass          = 10;      // total mass of the robot (kg)
        double std_f   = 0.001;   // standard deviation of the force sensors (N)
        double std_tau = 0.001;   // standard deviation of the torque sensors (N.m)

        ParamsSensorForceTorque() = default;
        ParamsSensorForceTorque(std::string _unique_name, const ParamsServer& _server):
            ParamsSensorBase(_unique_name, _server)
        {
            mass                 = _server.getParam<double>(_unique_name + "/mass");
            std_f          = _server.getParam<double>(_unique_name + "/std_f");
            std_tau        = _server.getParam<double>(_unique_name + "/std_tau");
        }
        ~ParamsSensorForceTorque() override = default;
        std::string print() const override
        {
            return ParamsSensorBase::print()                        + "\n"
                    + "mass: "          + std::to_string(mass)           + "\n"
                    + "std_f: "   + std::to_string(std_f)    + "\n"
                    + "std_tau: " + std::to_string(std_tau)  + "\n";
        }
};

WOLF_PTR_TYPEDEFS(SensorForceTorque);

class SensorForceTorque : public SensorBase
{

    protected:
        //noise std dev
        double mass_;            // total mass of the robot (kg)
        double std_f_;     // standard deviation of the force sensors (N)
        double std_tau_;   // standard deviation of the torque sensors (N.m)

    public:

        SensorForceTorque(const Eigen::VectorXd& _extrinsics, ParamsSensorForceTorquePtr _params);
        ~SensorForceTorque() override = default;

        WOLF_SENSOR_CREATE(SensorForceTorque, ParamsSensorForceTorque, 0);

        double getMass() const;
        double getForceNoiseStd() const;
        double getTorqueNoiseStd() const;
};

inline double SensorForceTorque::getMass() const
{
    return mass_;
}

inline double SensorForceTorque::getForceNoiseStd() const
{
    return std_f_;
}

inline double SensorForceTorque::getTorqueNoiseStd() const
{
    return std_tau_;
}

} // namespace wolf

#endif // SENSOR_FORCE_TORQUE_H
