//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
/**
 * \file gtest_capture_inertial_kinematics.cpp
 *
 *  Created on: Feb 28, 2020
 *      \author: jsola
 */


#include "bodydynamics/capture/capture_inertial_kinematics.h"
#include "bodydynamics/sensor/sensor_inertial_kinematics.h"

#include "core/utils/utils_gtest.h"
#include <core/utils/logging.h>

#include <Eigen/Dense>

using namespace wolf;

TEST(CaptureInertialKinematics, constructor_and_getters)
{
    SensorInertialKinematicsPtr sen;
    Vector9d data = Vector9d::Random();
    Matrix3d Iq = Matrix3d::Random();
    Vector3d Lcm = Vector3d::Random();
    Vector3d bias = Vector3d::Random();
    CaptureInertialKinematicsPtr C = std::make_shared<CaptureInertialKinematics>(0, sen, data, Iq, Lcm, bias);

    ASSERT_MATRIX_APPROX(C->getData(), data, 1e-20);
}

int main(int argc, char **argv)
{
  testing::InitGoogleTest(&argc, argv);
  return RUN_ALL_TESTS();
}

